import 'package:dio/dio.dart';

import '../../utils/log.dart';
import 'http.dart';

class HttpLogInterceptor extends Interceptor {
  HttpLogInterceptor();

  @override
  void onRequest(
    RequestOptions options,
    RequestInterceptorHandler handler,
  ) {
    Log.d('============================================================');
    Log.d("[${options.method}] ${options.uri.toString()}");
    if (Http.isFullLogEnable) {
      Log.d(options.headers);
      Log.d(options.data);
    }
    Log.d('------------------------------------------------------------');

    return handler.next(options);
  }

  @override
  void onResponse(Response response, ResponseInterceptorHandler handler) {
    Log.d('HTTP StatusCode: ${response.statusCode}');
    if (Http.isFullLogEnable) {
      Log.d(response.headers);
      Log.d(response.data);
    }
    Log.d('------------------------------------------------------------');

    return handler.next(response);
  }

  @override
  void onError(DioError err, ErrorInterceptorHandler handler) {
    Log.e('HTTP StatusCode: ${err.response?.statusCode}');
    Log.e(err.toString());
    if (Http.isFullLogEnable) {
      Log.e(err.response?.headers.toString());
      Log.e(err.response?.data);
    }
    Log.e('------------------------------------------------------------');

    return handler.next(err);
  }
}
